import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.http.HttpRequest.BodyPublishers;
import java.net.http.HttpRequest.Builder;
import java.net.http.HttpResponse.BodyHandlers;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.MessageFormat;
import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Base64;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

public class IssuetrakApiAuthorization {   
    private String _apiKey;
    private URI _baseUrl;
    
    public IssuetrakApiAuthorization(String baseUrl, String apiKey) throws URISyntaxException
    {
        _baseUrl = new URI(baseUrl);
        _apiKey = apiKey;
    }

    public CompletableFuture<Void> InvokeApiRequest(String httpVerb, String endpoint, String requestBody, String requestQuery)
    {   
        endpoint = endpoint.substring(0, 1) == "/" ? endpoint.substring(1) : endpoint;
        requestBody = requestBody != null && false == requestBody.trim().isEmpty() ? requestBody : "";
        requestQuery = requestQuery != null && false == requestQuery.trim().isEmpty() ? MessageFormat.format("?{0}", requestQuery) : "";
        
        var requestId = UUID.randomUUID().toString().toLowerCase();
        var formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSSSSSS'Z'").withZone(ZoneId.of("UTC"));
        var requestTimeStamp = formatter.format(Instant.now());
        var requestPath =
            MessageFormat.format(
            "{0}{1}",
            _baseUrl.resolve(endpoint),
            requestQuery != null && false == requestQuery.trim().isEmpty() ? requestQuery : "");
        var absoluteUrl = URI.create(requestPath).getPath().toLowerCase();
        
        var messageToHash = GenerateMessageToHash(httpVerb, requestId, requestTimeStamp, absoluteUrl, requestQuery, requestBody);
        var hashedMessage = GenerateRequestHash(messageToHash);
        
        return
            PerformHttpRequest(
                httpVerb,
                requestId,
                requestTimeStamp,
                hashedMessage,
                requestPath.toString(),
                requestBody
            );
    }
    
    private CompletableFuture<Void> PerformHttpRequest(
        String httpVerb, 
        String requestId, 
        String requestTimeStamp, 
        String messageHash, 
        String fullRequestPath, 
        String requestBody)
    {
        HttpClient client = HttpClient.newHttpClient();
        Builder builder = HttpRequest.newBuilder()
            .uri(URI.create(fullRequestPath))
            .header("Accept", "application/json")
            .header("X-IssueTrak-API-Request-ID", requestId)
            .header("X-IssueTrak-API-Timestamp", requestTimeStamp)
            .header("X-IssueTrak-API-Authorization", messageHash);

        switch (httpVerb) {
            case "POST":
                builder
                    .header("Content-type", "application/json")    
                    .POST(BodyPublishers.ofString(requestBody));
                break;
            case "PUT":
                builder
                    .header("Content-type", "application/json")    
                    .PUT(BodyPublishers.ofString(requestBody));
                break;
            default:
                break;            
        }

        HttpRequest request = builder.build();

        return client.sendAsync(request, BodyHandlers.ofString())
            .thenApply(HttpResponse<String>::body)
            .thenAccept(System.out::println);
    }

    private String GenerateMessageToHash(
        String httpVerb,
        String requestId,
        String requestTimeStamp,
        String absoluteUrl,
        String requestQuery, 
        String requestBody) 
    {
        return String.join("\n", new String[]{httpVerb, requestId, requestTimeStamp, absoluteUrl, requestQuery, requestBody});
    }
    
    private String GenerateRequestHash(String messageToBeHashed)
    {
        try {
            final byte[] hashKeyBytes = _apiKey.getBytes(StandardCharsets.UTF_8);
            Mac sha512Hmac = Mac.getInstance("HmacSHA512");
            SecretKeySpec keySpec = new SecretKeySpec(hashKeyBytes, "HmacSHA512");
            sha512Hmac.init(keySpec);
            byte[] macData = sha512Hmac.doFinal(messageToBeHashed.getBytes(StandardCharsets.UTF_8));
    
            return Base64.getEncoder().encodeToString(macData);
        } catch (NoSuchAlgorithmException | InvalidKeyException e) {
            e.printStackTrace();
        }

        return "";
    }
}